@@ -1,6 +1,7 @@
 from layout.LayoutObject import LayoutObject
 from config.DisplayConfigger import DisplayConfigger
 from config.StateSaver import DefaultStateSaver
+from DisplayConfigurator import DisplayConfigurator
 from DisplayTarget import DisplayTarget
 from TargetGroup import TargetGroup
 from MenuItem import MenuItem
@@ -10,10 +11,12 @@ from scripting.Scriptlet import Scriptlet
 from utils import actionparser
 from utils.Struct import Struct
 from utils.Observable import Observable
+from utils.HIGDialog import HIGDialog
 from layout import Unit
 from utils import vfs
-import utils
 
+import targetregistry
+import utils
 import utils.dialog
 
 import gobject
@@ -24,11 +27,9 @@ import sys
 import os
 import re
 
-#utils.cimport("utils.tiling", "Tiling")
 try:
     from utils.tiling import Tiling
 except ImportError:
-    import sys
     log("Could not import tiling module!")
     sys.exit(1)
 
@@ -92,7 +93,10 @@ class Display(gtk.HBox, Observable):
             MenuItem("/__restart", _("Re_start desklet"),
                      callback = self.__handle_restart),
             MenuItem("/__remove", _("_Remove desklet"),
-                     callback = self.__handle_remove)
+                     callback = self.__handle_remove),
+            MenuItem("/__sep3"),
+            MenuItem("/__about", _("_About"),
+                     callback = self.__handle_about)
             ]
 
         # the layout object
@@ -119,14 +123,48 @@ class Display(gtk.HBox, Observable):
         # the scripting environment
         self.__script = Script(ident, rep)
 
-        # the configurator object
-        self.__configurator = DisplayConfigger(ident)
-        self.__configurator.set_scripting_environment(self.__script)
-
         # the path of the .display file
         self.__path = os.path.dirname(rep)
         self.__display_file = rep
 
+        # the configurator object
+        self.__configurator = DisplayConfigger(ident, self.__path)
+        self.__configurator.set_scripting_environment(self.__script)
+
+        # the about window
+        self.__about = gtk.AboutDialog()
+        self.__about.set_wrap_license(True)
+        self.__about.connect("response", self.__about_response_callback)
+        self.__about.connect("close", self.__about_close_and_delete_callback)
+        self.__about.connect("delete_event",
+                              self.__about_close_and_delete_callback)
+
+        # the readme window
+        self.__readme = HIGDialog(buttons = (gtk.STOCK_CLOSE,
+                                             gtk.RESPONSE_CLOSE), self_destroy = False)
+        self.__readme.set_title("%s - README" % os.path.basename(rep))
+
+        sw = gtk.ScrolledWindow()
+        sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
+        sw.set_shadow_type(gtk.SHADOW_IN)
+        textview = gtk.TextView()
+        textview.set_editable(False)
+        textview.set_overwrite(False)
+        textview.set_left_margin(8)
+        textview.set_right_margin(8)
+        textview.set_wrap_mode(gtk.WRAP_WORD)
+        textview.set_cursor_visible(False)
+        textview.set_size_request(500, 300)
+        self.__readme_buffer = textview.get_buffer()
+        sw.add(textview)
+        sw.show()
+        textview.show()
+        self.__readme.vbox.add(sw)
+        self.__readme.connect("response", self.__readme_response_callback)
+        self.__readme.connect("close", self.__readme_close_and_delete_callback)
+        self.__readme.connect("delete_event",
+                              self.__readme_close_and_delete_callback)
+
         # the unique ID of this display
         self.__id = ident
 
@@ -145,7 +183,6 @@ class Display(gtk.HBox, Observable):
         # whether the display reacts on events
         self.__is_sensitive = True
 
-
         gtk.HBox.__init__(self)
 
 
@@ -198,6 +235,7 @@ class Display(gtk.HBox, Observable):
         gobject.idle_add(self.__group.unlock_geometry)
 
 
+
     #
     # Sets the content of the display.
     #
@@ -207,17 +245,112 @@ class Display(gtk.HBox, Observable):
 
 
 
+    #
+    # Sets the callbacks for the responses (about and readme dialogs).
+    #
+    def __about_response_callback(self, about, response_id):
+        if (response_id < 0):
+            self.__about.hide()
+            self.__about.emit_stop_by_name("response")
+        elif (response_id == 2003):
+            self.__open_readme()
+        elif (response_id == gtk.RESPONSE_CLOSE):
+            self.__about.hide()
+
+
+
+    def __about_close_and_delete_callback(self, about, event=None):
+
+        self.__about.hide()
+        return True
+
+
+
+    def __readme_response_callback(self, readme, response_id):
+
+        if (response_id == gtk.RESPONSE_CLOSE):
+            self.__readme.hide()
+
+
+
+    def __readme_close_and_delete_callback(self, readme, event=None):
+
+        self.__readme.hide()
+        return True
+
+
+
     #
     # Sets metadata.
     #
-    def set_metadata(self, author = "", version = "", name= "", preview = ""):
+    def set_metadata(self, metadata):
+
+        pbuf = None
+        fbuf = None
+        rbuf = None
+
+        preview = metadata.get("preview", "")
+        name = metadata.get("name", "")
+        version = metadata.get("version", "")
+        copyright = metadata.get("copyright", "")
+        comments = metadata.get("comments", "")
+        license = metadata.get("license", "")
+        website = metadata.get("website", "")
+        author = metadata.get("author", "")
+        category = metadata.get("category", "")
+        dependency = metadata.get("dependency", "")
+        description = metadata.get("description", "") + "\n"
 
         # a banner without the display's name would look crappy
         if (name):
-            if (preview): preview = self.get_full_path(preview)
+            if (preview):
+                preview = self.get_full_path(preview)
+                data = vfs.read_entire_file(preview)
+                loader = gtk.gdk.PixbufLoader()
+                loader.write(data, len(data))
+                loader.close()
+                pbuf = loader.get_pixbuf()
             self.__configurator.set_banner(preview,
-                                  "<big>%s</big> %s\n"
-                                  "<small>%s</small>" % (name, version, author))
+                                "<big>%s</big> %s\n"
+                                "<small>%s</small>" % (name, version, author))
+
+            # include the LICENSE file if available
+            for elem in ("COPYING", "LICENSE"):
+                try:
+                    fbuf = open(self.get_full_path(elem), "r")
+                except:
+                    pass
+
+            if (fbuf):
+                license = fbuf.read()
+            else:
+                log("Warning: COPYING or LICENSE not included in desklet \"%s %s\".\nPlease contact the author!" % (name, version))
+
+            # include the README file if available
+            try:
+                rbuf = open(self.get_full_path("README"), "r")
+            except:
+                log("README file not included in desklet!")
+
+            if (rbuf):
+                self.__readme_button = \
+                    self.__about.add_button(_("_Readme"), 2003)
+                self.__about.action_area.reorder_child(self.__readme_button,
+                                                       gtk.PACK_START)
+                self.__readme_buffer.set_text(rbuf.read())
+
+            # add comments to the description
+            if (comments): description = "%s\n%s\n" % (description, comments)
+
+            # feed the About Window with the Metadata
+            self.__about.set_name(name)
+            self.__about.set_version(version)
+            self.__about.set_copyright(copyright)
+            self.__about.set_comments(description)
+            if (license): self.__about.set_license(license)
+            self.__about.set_website(website)
+            self.__about.set_authors([author])
+            self.__about.set_logo(pbuf)
 
 
 
@@ -291,19 +424,15 @@ class Display(gtk.HBox, Observable):
 
         if (etype == self.EVENT_MOTION):
             x, y = args
-
             utils.request_idle_call(self.__queue_motion, x, y, w, h, False)
             return
 
-
         elif (etype == self.EVENT_LEAVE):
             utils.request_idle_call(self.__queue_motion, 0, 0, w, h, True)
             return
 
-
         elif (etype == self.EVENT_PRESS):
             button, x, y, counter = args
-
             self.__pointer_pos = (x, y)
             if (counter == 1):
                 action = DisplayTarget.ACTION_PRESS
@@ -313,10 +442,8 @@ class Display(gtk.HBox, Observable):
             event = Struct(button = button, _args = [button])
             actions.append((action, event))
 
-
         elif (etype == self.EVENT_RELEASE):
             button, x, y = args
-
             if (button == 1):
                 if (abs(lx - x) < 10 and abs(ly - y) < 10):
                     action = DisplayTarget.ACTION_CLICK
@@ -334,10 +461,8 @@ class Display(gtk.HBox, Observable):
             event = Struct(button = button, _args = [button])
             actions.append((action, event))
 
-
         elif (etype == self.EVENT_SCROLL):
             direction, x, y = args
-
             if (direction == gtk.gdk.SCROLL_UP):
                 direction = 0
             elif (direction == gtk.gdk.SCROLL_DOWN):
@@ -348,24 +473,21 @@ class Display(gtk.HBox, Observable):
             event = Struct(direction = direction, _args = [direction])
             actions.append((action, event))
 
-
         elif (etype == self.EVENT_KEY_PRESS):
             key, x, y = args
             action = DisplayTarget.ACTION_KEY_PRESS
             event = Struct(key = key)
             actions.append((action, event))
 
-
         elif (etype == self.EVENT_KEY_RELEASE):
             key, x, y = args
             action = DisplayTarget.ACTION_KEY_RELEASE
             event = Struct(key = key)
             actions.append((action, event))
 
-
         else:
             # what kind of event did we get there?
-            assert (False)  # never reached
+            import traceback; traceback.print_exc()
 
         for action, event in actions:
             self.__group.get_layout_object().send_action(
@@ -407,7 +529,9 @@ class Display(gtk.HBox, Observable):
     #
     # Returns the display.
     #
-    def _get_display(self): return self
+    def _get_display(self):
+
+        return self
 
 
 
@@ -423,12 +547,11 @@ class Display(gtk.HBox, Observable):
 
     def new_child(self, childtype, settings, children):
 
-        import targetregistry
         # we don't catch the KeyError here, but in the DisplayFactory
         try:
             self.__group = targetregistry.create(childtype, self)
         except KeyError, exc:
-            log(`exc`)
+            log("Error: %s\n" % `exc`)
         self.__group.get_widget().show()
         cid = settings["id"]
         self.add_target_to_script(cid, self.__group)
@@ -452,7 +575,6 @@ class Display(gtk.HBox, Observable):
 
         if (configurators):
             # support old deprecated sensor stuff
-            from DisplayConfigurator import DisplayConfigurator
             dconf = DisplayConfigurator(configurators)
 
         else:
@@ -460,6 +582,25 @@ class Display(gtk.HBox, Observable):
 
 
 
+    #
+    # Opens the about dialog for this display.
+    #
+    def __open_about(self):
+        assert (self.__about)
+
+        self.__about.show()
+
+
+    #
+    # Opens the readme dialog for this display.
+    #
+    def __open_readme(self):
+        assert (self.__readme)
+
+        self.__readme.show()
+
+
+
     #
     # Saves this display's position.
     #
@@ -481,6 +622,8 @@ class Display(gtk.HBox, Observable):
         self.__save_position()
 
         self.__configurator.destroy()
+        self.__about.destroy()
+        self.__readme.destroy()
         self.update_observer(self.OBS_CLOSED, self.__id)
         self.drop_observers()
 
@@ -541,7 +684,6 @@ class Display(gtk.HBox, Observable):
                 self.call_sensor(legacy_call, path, *legacy_args)
 
             else:
-                #src.notify_handle_action(True)
                 src._setp("event", event)
                 self.execute_callback_script(call, src)
 
@@ -566,14 +708,9 @@ class Display(gtk.HBox, Observable):
                                                          self.__action_stamp,
                                                    DisplayTarget.ACTION_MOTION,
                                                          Struct())
-            #self.__group.handle_action(DisplayTarget.ACTION_MOTION,
-            #                           ux, uy, Struct())
-            #self.__group.notify_handle_action(True)
             self.__group.detect_leave(self.__action_stamp)
-            #self.__action_stamp += 1
         else:
             self.__group.detect_leave(self.__action_stamp)
-            #self.__group.notify_handle_action(False)
 
 
 
@@ -583,7 +720,6 @@ class Display(gtk.HBox, Observable):
     def child_observer(self, src, cmd):
 
         if (cmd == src.OBS_GEOMETRY):
-            #src.get_layout_object().dump()
             x, y, w, h = self.__group.get_geometry()
             ux, uy = self.__group.get_user_geometry()[:2]
 
@@ -615,9 +751,8 @@ class Display(gtk.HBox, Observable):
                 new_x = x.as_px() + dx.as_px()
                 new_y = y.as_px() + dy.as_px()
 
-                #sx, sy = self.__layout_object.get_scaling()
-                self.__group.set_xml_prop("x", str(new_x))# / sx))
-                self.__group.set_xml_prop("y", str(new_y))# / sy))
+                self.__group.set_xml_prop("x", str(new_x))
+                self.__group.set_xml_prop("y", str(new_y))
 
 
 
@@ -712,6 +847,7 @@ class Display(gtk.HBox, Observable):
                 try:
                     data = vfs.read_entire_file(filename)
                 except:
+                    log("Couldn't read file %s", (filename,))
                     return
                 try:
                     loader.write(data, len(data))
@@ -783,11 +919,9 @@ class Display(gtk.HBox, Observable):
                     m.append(MenuItem("/item%d" % cnt, entry[0], None,
                                       callback, args))
                 cnt += 1
-            #end for
 
             utils.request_call(self.open_menu, m)
 
-
         self.__sensor_controls[ident] = sensor
         sensor.bind("output", self.__set_settings, ident)
         sensor.bind("menu", set_menu)
@@ -903,7 +1037,6 @@ class Display(gtk.HBox, Observable):
                     item.set_image(img)
                 except:
                     import traceback; traceback.print_exc()
-                    pass
 
             else:
                 item = gtk.MenuItem(entry.label)
@@ -931,7 +1064,6 @@ class Display(gtk.HBox, Observable):
 
             m.append(item)
             tree["/".join(entry.path.split("/"))] = item
-        #end for
 
         mainmenu.popup(None, None, None, 0, 0)
 
@@ -953,7 +1085,7 @@ class Display(gtk.HBox, Observable):
 
         from config import settings
 
-        os.system("%s \"%s\" & disown" % (settings.editor, self.__display_file))
+        os.system("%s \"%s\" & " % (settings.editor, self.__display_file))
 
 
     def __handle_restart(self, *args):
@@ -968,12 +1100,17 @@ class Display(gtk.HBox, Observable):
 
 
 
+    def __handle_about(self, *args):
+
+        self.__open_about()
+
+
+
     def close(self):
 
         def remove_display(*args):
             self.update_observer(self.OBS_CLOSE, self.__id)
 
-
         utils.dialog.question(None,
                               _("Do you really want to remove this desklet ?"),
                               _("This desklet will no longer be displayed "
@@ -991,3 +1128,4 @@ class Display(gtk.HBox, Observable):
         return "id%d%d" % (int(time.time() * 100), random.randrange(0xffff))
 
     make_id = staticmethod(make_id)
+
